﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition
Imports System.Collections.ObjectModel
Imports System.Drawing.Printing
Public Class Form1
   Private WithEvents moPrinterOverlay As PrinterInteractiveOverLay
   Private moPrinterPage As CustomPagePrinterLayer
   Private moPageLayoutMap As MapPrinterLayer
   Private moShapefile As ShapeFileFeatureLayer
   Private moScaleBarLayer As ScaleBarPrinterLayer
   Private moScaleLineLayer As ScaleLinePrinterLayer
   Private moFeatureExtent As RectangleShape

   Public Enum EnumPagePosition
      TopLeft = 1
      TopRight = 2
      LowerLeft = 3
      LowerRight = 4
   End Enum
   Public Enum EnumCenterType
      Horizontal = 1
      Vertical = 2
   End Enum

   Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
      'Me.WindowState = FormWindowState.Maximized

      CreateDefaultLayout()

      Dim sInstructions As String =
         "1. Notice that when the page layout map draws for the first time the highway labels draw." & ControlChars.CrLf & ControlChars.CrLf &
         "2. Click the 'Zoom In' button repeatedly (one or two times should do it)." & ControlChars.CrLf & ControlChars.CrLf &
         "3. An unhandled exception is generated." & ControlChars.CrLf & ControlChars.CrLf &
         "4. Edit the source code (function mCreateIconStyle) so that the IconImageScale for the Icon Style is something other than 1 (the default)." & ControlChars.CrLf & ControlChars.CrLf &
         "5. Now the IconStyle labels draw correctly, but they also draw outside the boundaries of the map. This becomes more noticeable as you zoom in farther."

      TextBox1.Text = sInstructions

   End Sub
   Private Function mCreateIconStyle() As IconStyle
      '***** Create an icon text style for the layer 
      Dim oIconTextStyle As New IconStyle("..\..\Data\Interstate_3-Digit.png", "HWY_SYMBOL", New GeoFont("Arial", 8, DrawingFontStyles.Bold), New GeoSolidBrush(GeoColor.SimpleColors.White))
      With oIconTextStyle
         .DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels
         .LabelAllLineParts = True
         .TextLineSegmentRatio = 1
         '.IconImageScale = 0.99
      End With
      Return oIconTextStyle
   End Function
   Public Sub AddMapLayer()

      moPageLayoutMap.MapUnit = GeographyUnit.DecimalDegree

      Dim sPath As String = "..\..\Data\Hwy.shp"

      Dim oIconTextStyle As IconStyle = mCreateIconStyle()

      moShapefile = New ShapeFileFeatureLayer(sPath, ShapeFileReadWriteMode.ReadOnly)
      moShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(LineStyles.CreateSimpleLineStyle(GeoColor.SimpleColors.Red, 2, False))
      moShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(oIconTextStyle)

      moShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
      moPageLayoutMap.Layers.Add(moShapefile)
      moShapefile.Open()

      moShapefile.Open()
      Dim oExtent As RectangleShape = moShapefile.GetBoundingBox
      oExtent.ScaleDown(30)
      moPageLayoutMap.MapExtent = oExtent
      moShapefile.Close()
   End Sub

   Public Sub CreateDefaultLayout()
      Dim oExtentRectangle As RectangleShape

      moPrinterOverlay = New PrinterInteractiveOverLay()
      With PageLayout
         '***** Setup the map unit, you want to always use feet or meters for printer layout maps
         .MapUnit = GeographyUnit.Feet
         '***** Create the default zoom levels for the layout map
         .ZoomLevelSet = New PrinterZoomLevelSet(.MapUnit, PrinterHelper.GetPointsPerGeographyUnit(.MapUnit))

         '***** Set the background color to gray so there is contrast with the white page
         .BackgroundOverlay.BackgroundBrush = New GeoSolidBrush(GeoColor.StandardColors.LightGray)

         '***** Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
         '***** The interactive overlay allows the layers to be interacted with
         .InteractiveOverlays.Add("PrinterOverlay", moPrinterOverlay)
         .InteractiveOverlays.MoveToBottom("PrinterOverlay")

         '***** Create the PagePrinterLayer which shows the page boundary and 
         '***** is the area the user will arrange all of the other layers on top of.
         moPrinterPage = New CustomPagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait)
         With moPrinterPage
            .Open()
            .BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)
            moPrinterOverlay.PrinterLayers.Add("PrinterPage", moPrinterPage)
            '***** Get the page's extent, scale it up by 10%
            oExtentRectangle = CType(RectangleShape.ScaleUp(.GetPosition(), 10), RectangleShape)
            '***** Set oExtentRectangle as the default current extent
         End With
         .CurrentExtent = oExtentRectangle

         '***** Set the minimum scale of the map to the last zoom level
         .MinimumScale = .ZoomLevelSet.ZoomLevel20.Scale

         '***** Add a container for the map layers
         AddMapLayerContainer()

         '***** Add a layer to the page layout
         AddMapLayer()

         mAddCenteredTitleLabel()

         '***** Add a scale line
         CreateDefaultScaleLine()

         '***** Add a scale bar
         CreateDefaultScaleBar()

         Try
            btnRefresh_Click(Nothing, Nothing)
         Catch ex As Exception

         End Try

      End With
      moPageLayoutMap.IsDrawing = True

   End Sub
   Private Sub mAddCenteredTitleLabel()
      Dim oLabelPrinterLayer As LabelPrinterLayer = New LabelPrinterLayer()
      Dim oPageBoundingbox As RectangleShape

      '***** Setup the text and the font..
      oLabelPrinterLayer.Text = "Here is a Sample Label"
      oLabelPrinterLayer.Font = New GeoFont("Arial", 2, DrawingFontStyles.Bold)
      oLabelPrinterLayer.TextBrush = New GeoSolidBrush(GeoColor.StandardColors.Black)

      '***** Set the labels position so that is it centered on the page one inch from the top                        
      oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
      Dim oLabelCenter As PointShape = New PointShape()
      oLabelCenter.X = oPageBoundingbox.UpperRightPoint.X - oPageBoundingbox.Width / 2
      oLabelCenter.Y = oPageBoundingbox.UpperLeftPoint.Y - 0.75

      oLabelPrinterLayer.PrinterWrapMode = PrinterWrapMode.AutoSizeText
      oLabelPrinterLayer.SetPosition(5, 0.5, oLabelCenter, PrintingUnit.Inch)
      oLabelPrinterLayer.ResizeMode = PrinterResizeMode.Resizable


      moPrinterOverlay.PrinterLayers.Add(Guid.NewGuid.ToString, oLabelPrinterLayer)

   End Sub


   Public Sub AddMapLayerContainer()
      Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)

      moPageLayoutMap = New MapPrinterLayer()

      moPageLayoutMap.MapUnit = GeographyUnit.DecimalDegree

      '***** Create the MapPrinterLayer and set the position
      moPageLayoutMap.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)

      '***** Set the maps position slightly below the pages center and 7.5 inches wide and 9 inches tall
      oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
      moPageLayoutMap.SetPosition(7.5, 9, oPageBoundingbox.GetCenterPoint().X, oPageBoundingbox.GetCenterPoint().Y - 0.25, PrintingUnit.Inch)

      '***** Add the MapPrinterLayer to the PrinterInteractiveOverlay
      moPrinterOverlay.PrinterLayers.Add("MapPrinterLayer", moPageLayoutMap)
   End Sub
   Private Function GetPageBoundingBox(ByVal unit As PrintingUnit) As RectangleShape
      Return moPrinterPage.GetPosition(unit)
   End Function
   Public Sub CreateDefaultScaleLine()
      If moPrinterOverlay.PrinterLayers.Contains("ScaleLine") Then
         moPrinterOverlay.PrinterLayers.Remove("ScaleLine")
      End If

      '***** Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
      moScaleLineLayer = New ScaleLinePrinterLayer(moPageLayoutMap)

      moScaleLineLayer.MapUnit = moPageLayoutMap.MapUnit

      moScaleLineLayer.Open()
      mSetPosition(2.5, 0.25, moScaleLineLayer, EnumPagePosition.LowerLeft)

      '***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
      moPrinterOverlay.PrinterLayers.Add("ScaleLine", moScaleLineLayer)
   End Sub
   Public Sub CreateDefaultScaleBar()
      If moPrinterOverlay.PrinterLayers.Contains("ScaleBar") Then
         moPrinterOverlay.PrinterLayers.Remove("ScaleBar")
      End If

      '***** Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
      moScaleBarLayer = New ScaleBarPrinterLayer(moPageLayoutMap)

      With moScaleBarLayer
         .MapUnit = moPageLayoutMap.MapUnit
         .AlternateBarBrush = New GeoSolidBrush(GeoColor.SimpleColors.Black)
         .TextColor = GeoColor.SimpleColors.Black
         .Font = New GeoFont("Arial", 10)
         .BarBrush = New GeoSolidBrush(GeoColor.SimpleColors.BrightOrange)
         .UnitFamily = UnitSystem.Imperial
         .MaxWidth = 7
      End With

      moScaleBarLayer.Open()
      mSetPosition(2.5, 0.25, moScaleBarLayer, EnumPagePosition.LowerRight)

      '***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
      moPrinterOverlay.PrinterLayers.Add("ScaleBar", moScaleBarLayer)
   End Sub
   Public Sub mSetPosition(ByVal dWidth As Double, ByVal dHeight As Double, ByVal oPrinterLayer As PrinterLayer, ByVal eInitialPosition As EnumPagePosition)
      Dim dCenterPointX As Double
      Dim dCenterPointY As Double
      Dim dDistanceFromEdge As Double = 0.25

      '***** Get the bounding box for the MapPrinterLayer
      moPageLayoutMap.Open()
      Dim oMapBoundingBox As RectangleShape = moPageLayoutMap.GetPosition(PrintingUnit.Inch)
      Select Case eInitialPosition
         Case EnumPagePosition.LowerRight
            dCenterPointX = oMapBoundingBox.LowerRightPoint.X - (dWidth / 2) - dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.LowerRightPoint.Y + (dHeight / 2) + dDistanceFromEdge
         Case EnumPagePosition.LowerLeft
            dCenterPointX = oMapBoundingBox.LowerLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.LowerLeftPoint.Y + (dHeight / 2) + dDistanceFromEdge
         Case EnumPagePosition.TopRight
            dCenterPointX = oMapBoundingBox.UpperRightPoint.X - (dWidth / 2) - dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.UpperRightPoint.Y - (dHeight / 2) - dDistanceFromEdge
         Case EnumPagePosition.TopLeft
            dCenterPointX = oMapBoundingBox.UpperLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.UpperLeftPoint.Y - (dHeight / 2) - dDistanceFromEdge
      End Select

      '***** Now let's try to place this layer in the lower right corner of the page
      oPrinterLayer.SetPosition(dWidth, dHeight, dCenterPointX, dCenterPointY, PrintingUnit.Inch)
   End Sub

   Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
      mRefreshMapLayers()
   End Sub

   Private Sub mRefreshMapLayers()
      moPageLayoutMap.IsDrawing = True
      PageLayout.Refresh()
      'moPageLayoutMap.IsDrawing = False
   End Sub
   Private Sub btnZoomOut_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnZoomOut.Click
      Dim oExtent As RectangleShape = moPageLayoutMap.MapExtent
      oExtent = ExtentHelper.ZoomOut(oExtent, 10)
      moPageLayoutMap.MapExtent = oExtent
      mRefreshMapLayers()
   End Sub

   Private Sub btnZoomIn_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnZoomIn.Click
      Dim oExtent As RectangleShape = moPageLayoutMap.MapExtent
      oExtent = ExtentHelper.ZoomIn(oExtent, 10)
      moPageLayoutMap.MapExtent = oExtent
      mRefreshMapLayers()
   End Sub

End Class
